/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap;

/**
 * Compares {@link LRUMapCache} with {@link RandomMapCache}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class RandomVsLRUDemo {
   private static int getSetSize = 256;
   private static String[] keys = new String[1280];
   private static Map treeMap = new TreeMap();
   private static LRUMapCache lruCache = new LRUMapCache(treeMap, 128);
   private static RandomMapCache ramdomCache = new RandomMapCache(treeMap, 128);
   public static void main(String[] args) {
      fillMaps();
      int iterations = 100000;
      long start;

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readTreeMap();
      System.out.println("readTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readTreeMap();
      System.out.println("readTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readRandomTreeMap();
      System.out.println("readRandomTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readRandomTreeMap();
      System.out.println("readRandomTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readLRUTreeMap();
      System.out.println("readLRUTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readLRUTreeMap();
      System.out.println("readLRUTreeMap() " + (System.currentTimeMillis()-start));

      System.out.println("lruCache.getHitRatio() " + lruCache.getHitRatio());
      System.out.println("ramdomCache.getHitRatio() " + ramdomCache.getHitRatio());
   }

   public static void readLRUTreeMap() {
      for (int i=0; i<getSetSize; i++) {
         lruCache.get(keys[i]);
      }
   }

   public static void readRandomTreeMap() {
      for (int i=0; i<getSetSize; i++) {
         ramdomCache.get(keys[i]);
      }
   }

   public static void readTreeMap() {
      for (int i=0; i<getSetSize; i++) {
         treeMap.get(keys[i]);
      }
   }

   private static void fillMaps() {
      // fill
      for (int i=0; i<keys.length; i++) {
         keys[i] = "key" + i;
         treeMap.put(keys[i], "value");
      }
   }


}
