/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;

/**
 * Reads numbers from a file, sorts them and writes them to another file.
 * This implementation uses a {@link IntegerSortPipe}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TelephoneNumberSort5 {

   public static void main(String[] args) throws IOException {
      sort(args[0]);
      System.gc();
      sort(args[0]);
   }

   private static void sort(String filename) throws IOException {
      long start = System.currentTimeMillis();
      FileChannel in = new FileInputStream(filename).getChannel();
      int size = (int) in.size();
      int toTransfer = size;
      IntegerSortPipe bsb = new IntegerSortPipe(8999999, 1000000);
      while (toTransfer > 0) {
         toTransfer -= in.transferTo(size-toTransfer, toTransfer, bsb);
      }
      in.close();
      FileChannel out = new FileOutputStream(filename + ".sorted").getChannel();
      toTransfer = size;
      while (toTransfer > 0) {
         toTransfer -= out.transferFrom(bsb, size-toTransfer, toTransfer);
      }
      out.close();
      System.out.println(System.currentTimeMillis() - start + "ms");
      bsb.close();
   }

}
