/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.exceptions;

import java.util.Set;

/**
 * Demonstrates the performance effects of re-usedexceptions.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ReusedExceptionDemo {

   private static final Exception REUSED_EXCEPTION = new Exception("Stacktrace is invalid.");

   public static void main(String[] args) {
      int iterations = 100;
      long start;
      long time;

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalException();
      time = System.currentTimeMillis() - start;
      System.out.println("normalException() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalException();
      time = System.currentTimeMillis() - start;
      System.out.println("normalException() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) reusedException();
      time = System.currentTimeMillis() - start;
      System.out.println("reusedException() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) reusedException();
      time = System.currentTimeMillis() - start;
      System.out.println("reusedException() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) reusedExceptionFilledInStacktrace();
      time = System.currentTimeMillis() - start;
      System.out.println("reusedExceptionFilledInStacktrace() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) reusedExceptionFilledInStacktrace();
      time = System.currentTimeMillis() - start;
      System.out.println("reusedExceptionFilledInStacktrace() second run: " + time);
   }

   private static void normalException() {
      for (int i=0; i<10000; i++) {
         try {
            throw new Exception();
         }
         catch (Exception e) {
         }
      }
   }

   private static void reusedException() {
      for (int i=0; i<10000; i++) {
         try {
            throw REUSED_EXCEPTION;
         }
         catch (Exception e) {
         }
      }
   }

   private static void reusedExceptionFilledInStacktrace() {
      for (int i=0; i<10000; i++) {
         try {
            REUSED_EXCEPTION.fillInStackTrace();
            throw REUSED_EXCEPTION;
         }
         catch (Exception e) {
         }
      }
   }
}
