/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.exceptions;

/**
 * Demonstrates the performance effects of try-catch-blocks in loops.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TryCatchInLoopDemo {
   public static void main(String[] args) {
      int iterations = 100;
      long start;
      long time;

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) loopWithoutTryCatch();
      time = System.currentTimeMillis() - start;
      System.out.println("loopWithoutTryCatch() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) loopWithoutTryCatch();
      time = System.currentTimeMillis() - start;
      System.out.println("loopWithoutTryCatch() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) loopWithTryCatch();
      time = System.currentTimeMillis() - start;
      System.out.println("loopWithTryCatch() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) loopWithTryCatch();
      time = System.currentTimeMillis() - start;
      System.out.println("loopWithTryCatch() second run: " + time);
   }

   private static int loopWithoutTryCatch() {
      int a = 0;
      for (int i=0; i<100000; i++) {
         a += Integer.parseInt("1");
      }
      return a;
   }

   private static int loopWithTryCatch() {
      int a = 0;
      for (int i=0; i<100000; i++) {
         try {
            a += Integer.parseInt("1");
         }
         catch (NumberFormatException nfe) {
            nfe.printStackTrace();
         }
      }
      return a;
   }

}
