/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.httpd;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.ServerSocketChannel;

/**
 * Http daemon that uses the new I/O api. Connections are accepted through
 * {@link Acceptor} threads and as {@link Connection}s through a queue registered with
 * a {@link java.nio.channels.Selector}. {@link ConnectionSelector} threads call the
 * connection's work methods until they de-register themselves.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class NIOHttpd {

   /**
    * Guesses the content type.
    */
   public static String guessContentType(String uri) {
      uri = uri.toLowerCase();
      if (uri.endsWith(".html") || uri.endsWith(".htm")) {
         return "text/html";
      } else if (uri.endsWith(".txt")) {
         return "text/plain";
      } else if (uri.endsWith(".jpg") || uri.endsWith(".jpeg")) {
         return "image/jpeg";
      } else if (uri.endsWith(".gif")) {
         return "image/gif";
      } else {
         return "unknown";
      }
   }

   /**
    * Starts the Http daemon with 2args[0] threads.
    */
   public static void main(String[] args) throws IOException {
      ServerSocketChannel serverSocketChannel = ServerSocketChannel.open();
      serverSocketChannel.socket().bind(new InetSocketAddress(8080));
      for (int i = 0; i < Integer.parseInt(args[0]); i++) {
         ConnectionSelector connectionSelector = new ConnectionSelector();
         Acceptor acceptor = new Acceptor(serverSocketChannel, connectionSelector);
      }
   }

}
