/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

/**
 * Demonstrates different methods of leaving a loop.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class BreakingLoopDemo {
   public static void main(String[] args) {
      int iterations = 10;
      long start;
      long time;

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) booleanTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("booleanTerminatedLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) booleanTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("booleanTerminatedLoop() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) breakTerminatedLoop();
       time = System.currentTimeMillis() - start;
      System.out.println("breakTerminatedLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) breakTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("breakTerminatedLoop() second run: " + time);

   }

   private static int breakTerminatedLoop() {
      int a=0;
      for (int i=0; i<20000000; i++) {
         a += i;
         if (a == -1) break;
      }
      return a;
   }

   private static int booleanTerminatedLoop() {
      int a=0;
      boolean done = false;
      for (int i=0; i<20000000 && !done; i++) {
         a += i;
         if (a == -1) done = true;
      }
      return a;
   }

}
