/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

/**
 * Demonstrates manual loop unrolling.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class LoopUnrollingDemo {
   public static void main(String[] args) {
      int iterations = 100;
      long start;
      long time;

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("normalLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("normalLoop() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) unrolledLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("unrolledLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) unrolledLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("unrolledLoop() second run: " + time);
   }

   private static int normalLoop() {
      int a=0;
      for (int i=0; i<20000000; i++) {
         a += i;
      }
      return a;
   }

   private static int unrolledLoop() {
      int a=0;
      for (int i=0; i<20000000; i+=10) {
         a += i;
         a += i+1;
         a += i+2;
         a += i+3;
         a += i+4;
         a += i+5;
         a += i+6;
         a += i+7;
         a += i+8;
         a += i+9;
      }
      return a;
   }
}
