/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * Demonstrates a reflective action switch.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ReflectiveActionSwitchDemo {

   private static class ReflectiveAction implements Action {
      private Method method;
      private Object targetObject;

      public ReflectiveAction(Object targetObject, String methodName) throws NoSuchMethodException {
         this.targetObject = targetObject;
         method = targetObject.getClass().getMethod(methodName, null);
      }

      public void action() {
         try {
            method.invoke(targetObject, null);
         } catch (InvocationTargetException ita) {
            Throwable t = ita.getTargetException();
            if (t instanceof RuntimeException) throw (RuntimeException) t;
            if (t != null)
               t.printStackTrace();
            else
               ita.printStackTrace();
         } catch (IllegalAccessException iae) {
            iae.printStackTrace();
         }

      }
   }

   private Map actionMap;

   public ReflectiveActionSwitchDemo() throws NoSuchMethodException {
      actionMap = new HashMap();
      actionMap.put("Action1", new ReflectiveAction(this, "action1"));
      actionMap.put("Action2", new ReflectiveAction(this, "action2"));
      actionMap.put("Action3", new ReflectiveAction(this, "action3"));
   }

   public void action1() {
      System.out.println("Action1");
   }

   public void action2() {
      System.out.println("Action2");
   }

   public void action3() {
      System.out.println("Action3");
   }

   public void performAction(String s) {
      Action action = (Action) actionMap.get(s);
      if (action != null) action.action();
   }

   public static void main(String[] args) throws NoSuchMethodException {
      ReflectiveActionSwitchDemo asd = new ReflectiveActionSwitchDemo();
      asd.performAction("Action2");
   }

}
