/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

/**
 * Demonstrates the two different kinds of switches (tableswitch and lookupswitch).
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class SwitchDemo {
   public static void main(String[] args) {
      int iterations = 1000000;
      long start;
      long time;

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) twoSwitches();
      time = System.currentTimeMillis() - start;
      System.out.println("twoSwitches() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) twoSwitches();
      time = System.currentTimeMillis() - start;
      System.out.println("twoSwitches() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) twoSwitches();
      time = System.currentTimeMillis() - start;
      System.out.println("twoSwitches() third run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) oneSwitch();
       time = System.currentTimeMillis() - start;
      System.out.println("oneSwitch() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) oneSwitch();
      time = System.currentTimeMillis() - start;
      System.out.println("oneSwitch() second run: " + time);

   }

   private static int oneSwitch() {
      int defaultValue = 99;
      int a = 0;
      for (int i=0; i<120; i++) {
         switch(i) {
            case 0: a=0; break;
            case 1: a=defaultValue; break;
            case 2: a=defaultValue; break;
            case 3: a=3; break;
            case 4: a=4; break;
            case 5: a=defaultValue; break;
            case 6: a=6; break;
            case 7: a=defaultValue; break;
            case 8: a=defaultValue; break;
            case 9: a=9; break;

            case 100: a=0; break;
            case 101: a=defaultValue; break;
            case 102: a=defaultValue; break;
            case 103: a=3; break;
            case 104: a=4; break;
            case 105: a=defaultValue; break;
            case 106: a=6; break;
            case 107: a=defaultValue; break;
            case 108: a=defaultValue; break;
            case 109: a=9; break;
            default: a=defaultValue;
         }
      }
      return a;
   }

   private static int twoSwitches() {
      int defaultValue = 99;
      int a = 0;
      for (int i=0; i<120; i++) {
         switch(i) {
            case 0: a=0; break;
            case 1: a=defaultValue; break;
            case 2: a=defaultValue; break;
            case 3: a=3; break;
            case 4: a=4; break;
            case 5: a=defaultValue; break;
            case 6: a=6; break;
            case 7: a=defaultValue; break;
            case 8: a=defaultValue; break;
            case 9: a=9; break;
         }
         switch(i) {
            case 100: a=0; break;
            case 101: a=defaultValue; break;
            case 102: a=defaultValue; break;
            case 103: a=3; break;
            case 104: a=4; break;
            case 105: a=defaultValue; break;
            case 106: a=6; break;
            case 107: a=defaultValue; break;
            case 108: a=defaultValue; break;
            case 109: a=9; break;
            default: a=defaultValue;
         }
      }
      return a;
   }

}
