/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import com.tagtraum.perf.datastructures.RandomCache;

import java.io.*;
import java.util.Random;

/**
 * Demonstrates the {@link FileCache}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class FileCacheDemo {


   private static Random rnd = new Random();
   private static byte[] buf = new byte[1024];
   private static byte[] readBuf = new byte[1024 * 8];
   private static int iterations;

   public static void main(String[] args) throws Exception {
      rnd.nextBytes(buf);
      File[] file = new File[4];
      for (int i = 0; i < file.length; i++) {
         file[i] = makeFile((int) Math.pow(10, i));
      }
      for (iterations = 10; iterations < 10000; iterations *= 10) {
         System.out.println();
         System.out.println("Iterations: " + iterations);
         for (int i = 0; i < file.length; i++) {
            long notCached = 0;
            long cached = 0;
            int runIt = 10000 / iterations;
            for (int j = 0; j < runIt; j++) {
               FileCache rc = new FileCache(new RandomCache(10));
               long start = System.currentTimeMillis();
               readFile(file[i]);
               notCached = System.currentTimeMillis() - start;

               start = System.currentTimeMillis();
               readCachedFile(rc, file[i]);
               cached = System.currentTimeMillis() - start;
            }
            System.gc();
            System.out.println("not-cached " + (int) Math.pow(10, i) + ": " + (notCached/runIt));
            System.out.println("cached     " + (int) Math.pow(10, i) + ": " + (cached/runIt));
         }
      }
   }

   private static void readCachedFile(FileCache rc, File file) throws IOException {
      for (int i = 0; i < iterations; i++) {
         FileCache.Entry r = rc.get(file);
         InputStream in = r.getNewInputStream();
         while (in.read(readBuf) != -1) {
         }
         in.close();
      }
   }

   private static void readFile(File file) throws IOException {
      for (int i = 0; i < iterations; i++) {
         InputStream in = new BufferedInputStream(new FileInputStream(file));
         while (in.read(readBuf) != -1) {
         }
         in.close();
      }
   }

   private static File makeFile(int sizeInKB) throws IOException {
      System.out.println("Writing file with size " + (sizeInKB * 1024));
      File file = File.createTempFile("ResourceCacheDemo", ".tmp");
      FileOutputStream out = new FileOutputStream(file);
      for (int i = 0; i < sizeInKB; i++) out.write(buf);
      out.close();
      return file;
   }
}
