/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.memory;

/**
 * This class simply allocates some memory in a member array,
 * a local String array, a two-diemnsional String array, and
 * a two-dimensional byte array.
 * <p>
 * It should be runwith the Sun Java VM option
 * <code>-Xrunhprof</code> and the resulting file java.hprof.txt
 * should be inspected.<p>
 * On Linux systems you might need to set the flag <code>-classic</code> or
 * <code>-green -Djava.compiler=</code> to turn off HotSpot and use green
 * threads instead of native threads. It has been observed that native
 * threads on linux lead to somewhat strange results.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class HprofHeapDemo2 {

   private String[] stringArray;
   private String[][] twoDimensionalStringArray;
   private byte[][] twoDimensionalByteArray;
   private byte[][][] threeDimensionalByteArray;
   private String veryLongString;

   public void allocateStrings() {
      stringArray = new String[1024];
      for (int i=0; i<stringArray.length; i++) {
         stringArray[i] = Integer.toString(i);
      }
   }

   public void temporaryAllocateStringArray() {
      String[] localStringArray = new String[1024];
   }

   public void allocateTwoDimensionalStringArray() {
      twoDimensionalStringArray = new String[512][512];
   }

   public void allocateTwoDimensionalByteArray() {
      twoDimensionalByteArray = new byte[512][512];
   }

   public void allocateThreeDimensionalByteArray() {
      threeDimensionalByteArray = new byte[16][16][16];
   }

   public void allocateVeryLongString() {
      char[] c = new char[1024 * 10];
      veryLongString = new String(c);
   }

   public static void main(String[] args) {
      HprofHeapDemo2 demoInstance = new HprofHeapDemo2();
      demoInstance.allocateStrings();
      for (int i=0; i<2048; i++) {
         demoInstance.temporaryAllocateStringArray();
      }
      demoInstance.allocateTwoDimensionalByteArray();
      demoInstance.allocateThreeDimensionalByteArray();
      demoInstance.allocateTwoDimensionalStringArray();
      demoInstance.allocateVeryLongString();
   }
}
