/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.memviewer;

import java.util.Date;

/**
 * Snapshot of the current state of the memory.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class MemState {
   private long totalMemory;
   private long freeMemory;
   private long time;

   public MemState() {
      this.totalMemory = Runtime.getRuntime().totalMemory();
      this.freeMemory = Runtime.getRuntime().freeMemory();
      this.time = System.currentTimeMillis();
   }

   public long getTotalMemory() {
      return totalMemory;
   }

   public long getFreeMemory() {
      return freeMemory;
   }

   public long getUsedMemory() {
      return totalMemory - freeMemory;
   }

   public long getUsedMemoryPercent() {
      return getUsedMemory() * 100 / getTotalMemory();
   }

   public long getFreeMemoryPercent() {
      return getFreeMemory() * 100 / getTotalMemory();
   }

   public long getTime() {
      return time;
   }

   public String toString() {
      return new Date(getTime()).toString() + ": Total: " + getTotalMemory()
            + ", Free: " + getFreeMemory() + " (" + getFreeMemoryPercent() + "%)"
            + ", Used: " + getUsedMemory() + " (" + getUsedMemoryPercent() + "%)";
   }
}
