/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.net;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.zip.GZIPInputStream;

/**
 * Reads a URL and tries to use gzip content decoding.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class AcceptGZIPDemo {
   public static void main(String[] args) throws Exception {
      URL url = new URL(args[0]);
      HttpURLConnection connection = (HttpURLConnection) url.openConnection();
      connection.addRequestProperty("Accept-Encoding", "gzip");
      InputStream in = connection.getInputStream();
      String contentEncoding = connection.getHeaderField("Content-Encoding");
      if (contentEncoding != null && contentEncoding.equals("gzip")) {
         in = new GZIPInputStream(in);
         System.err.println("GZIP!");
      }
      byte[] buf = new byte[1024];
      int length;
      while ((length = in.read(buf)) != -1) {
         System.out.println(new String(buf, 0, length));
      }
      in.close();
   }

}
