/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.rmi;

import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

/**
 * RMI client that repeatedly calls a server method with different arguments.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ClientDemo {
   private static ServerDemo serverDemo = null;
   private static String[] items = new String[1000];
   public static void main(String[] args) throws Exception {
      getServerDemo();
      for (int i=0; i<items.length; i++) {
         items[i] = "String " + i;
      }
      for (int i=1; i<items.length*10; i*=10) {
         long time = 0;
         double iterations = 40;
         for (int j=0; j<iterations; j++) {
            long start = System.currentTimeMillis();
            callSend(items.length/i);
            time += System.currentTimeMillis() - start;
         }
         System.out.println(i + ": " +(time/iterations) + "ms");
      }
   }

   private static void callSend(int bucketSize) throws RemoteException {
      for (int call=0; call <items.length / bucketSize; call++) {
         String[] bucket = new String[bucketSize];
         System.arraycopy(items, call*bucketSize, bucket, 0, bucketSize);
         serverDemo.send(bucket);
      }
   }

   private static void getServerDemo() throws NotBoundException, java.net.MalformedURLException, RemoteException {
      serverDemo = (ServerDemo) Naming.lookup("//localhost/ServerDemo");
   }
}
