/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.serialization;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * InternationalDate implementation with transient map and date.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class InternationalDate3 implements Serializable {
   private transient Date date;
   private transient Map map;

   public InternationalDate3() {
      this(System.currentTimeMillis());
   }

   public InternationalDate3(long time) {
      date = new Date(time);
      buildInternationalStrings();
   }

   private void buildInternationalStrings() {
      map = new HashMap();
      Locale locales[] = Locale.getAvailableLocales();
      for (int i = 0; i < InternationalDateDemo.availableLocales; i++) {
         SimpleDateFormat format = new SimpleDateFormat("EEEE d MMM yyyy G HH:mm:ss,SSS zzzz", locales[i]);
         String formattedTime = format.format(date);
         map.put(locales[i], formattedTime);
      }
   }

   public Date getDate() {
      return date;
   }

   public String get(Locale locale) {
      return (String) map.get(locale);
   }

   private void writeObject(ObjectOutputStream out) throws IOException {
      out.writeLong(date.getTime());
   }

   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
      date = new Date(in.readLong());
      buildInternationalStrings();
   }

}
