/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.serialization;

import com.tagtraum.perf.datastructures.Cache;
import com.tagtraum.perf.datastructures.RandomCache;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * InternationalDate implementation that uses a a cache.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class InternationalDate5 implements Serializable {
   private transient Date date;
   private transient Map map;
   private static Cache cache = new RandomCache(512);

   private InternationalDate5(Date date) {
      this.date = date;
      buildInternationalStrings();
   }

   private void buildInternationalStrings() {
      map = new HashMap();
      Locale locales[] = Locale.getAvailableLocales();
      for (int i = 0; i < InternationalDateDemo.availableLocales; i++) {
         SimpleDateFormat format = new SimpleDateFormat("EEEE d MMM yyyy G HH:mm:ss,SSS zzzz", locales[i]);
         String formattedTime = format.format(date);
         map.put(locales[i], formattedTime);
      }
   }

   public Date getDate() {
      return date;
   }

   public String get(Locale locale) {
      return (String) map.get(locale);
   }

   public static synchronized InternationalDate5 getInstance(Date date) {
      InternationalDate5 iDate = (InternationalDate5) cache.get(date);
      if (iDate == null) {
         iDate = new InternationalDate5(date);
         cache.put(date, iDate);
      }
      return iDate;
   }

   public static synchronized InternationalDate5 getInstance() {
      return getInstance(new Date());
   }

   private Object readResolve() throws ObjectStreamException {
      return getInstance(date);
   }

   private void writeObject(ObjectOutputStream out) throws IOException {
      out.writeLong(date.getTime());
   }

   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
      date = new Date(in.readLong());
   }

}
