/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.serialization;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * Demonstrates the serialization behaviour of different LinkedList classes.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 * @see LinkedList1
 * @see LinkedList2
 * @see LinkedList3
 */
public class LinkedListDemo {

   public static void main(String[] args) throws Exception {
      int start = 700;
      int end = 710;
      for (int length = start; length < end; length += 10) {
         testList(LinkedList1.class, length);
      }
      for (int length = start; length < end; length += 10) {
         testList(LinkedList2.class, length);
      }
      for (int length = start; length < end; length += 10) {
         testList(LinkedList3.class, length);
      }
   }

   private static void testList(Class klass, int length) throws Exception {
      LinkedList list = buildLinkedList(klass, length);
      double iterations = 1000;
      long saveTime = 0;
      long loadTime = 0;
      int size = 0;
      for (int i = 0; i < iterations; i++) {
         ByteArrayOutputStream bout = new ByteArrayOutputStream();
         ObjectOutputStream out = new ObjectOutputStream(bout);
         long start = System.currentTimeMillis();
         out.writeObject(list);
         out.close();
         saveTime += System.currentTimeMillis() - start;
         size = bout.size();
         ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
         ObjectInputStream in = new ObjectInputStream(bin);
         start = System.currentTimeMillis();
         in.readObject();
         in.close();
         loadTime += System.currentTimeMillis() - start;
      }
      System.out.println(klass + " " + length + " " + size + "bytes save: " + (saveTime / iterations) + "ms load: " + (loadTime / iterations) + "ms");
   }

   private static LinkedList buildLinkedList(Class klass, int length) throws Exception {
      LinkedList list = (LinkedList) klass.newInstance();
      for (int i = 0; i < length; i++) {
         list.add("Element " + i);
      }
      return list;
   }
}
