/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

/**
 * Class that demonstrates Inserts with Strings or StringBuffers.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class StringInsertDemo {
    private static String halloWelt = "Hallo Welt ";
    private static String weite = "weite ";

    public static void main(String[] args) {
        int iterations = 4000000;
        long start;
        long time;

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStrings();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStrings first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStrings();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStrings second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStringBuffers first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStringBuffers second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStringsAndPlus();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStringsAndPlus first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) insertWithStringsAndPlus();
        time = System.currentTimeMillis() - start;
        System.out.println("insertWithStringsAndPlus second run: " + time);
    }

    private static void insertWithStrings() {
        // insert only with Strings and concat
        String hallo = halloWelt.substring(0, 7);
        String welt = halloWelt.substring(7);
        String halloWeiteWelt = hallo.concat(weite).concat(welt);
    }

    private static void insertWithStringBuffer() {
        // insert with StringBuffer
        StringBuffer stringBuffer = new StringBuffer(halloWelt);
        stringBuffer.insert(7, weite);
        String halloWeiteWelt = stringBuffer.toString();
    }

    private static void insertWithStringsAndPlus() {
        // insert only with Strings and +
        String hallo = halloWelt.substring(0, 7);
        String welt = halloWelt.substring(7);
        String halloWeiteWelt = hallo + weite + welt;
    }

}
