/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.util.*;
import java.text.Collator;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;

/**
 * Demonstrates sorting of strings in different ways.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class StringSortDemo {
    private static List words;
    private static Collator collator = Collator.getInstance(Locale.GERMAN);

    public static void main(String[] args) throws IOException {
        showNaturalOrderExample();

        words = new ArrayList();
        for (int i=0; i<250; i++) fillWordList(args[0]); // 250
        System.out.println("words: " + words.size());
        long start;
        long time;

        Object[] freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sort(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sort first run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sort(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sort second run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCaseInsensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCaseInsensitive first run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCaseInsensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCaseInsensitive second run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCollator(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCollator first run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCollator(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCollator second run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCombinedCaseInsensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCombinedCaseInsensitive first run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCombinedCaseInsensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCombinedCaseInsensitive second run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCombinedCaseSensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCombinedCaseSensitive first run: " + time);

        freshList = (Object[])words.toArray(new String[0]);
        start = System.currentTimeMillis();
        sortCombinedCaseSensitive(freshList);
        time = System.currentTimeMillis() - start;
        System.out.println("sortCombinedCaseSensitive second run: " + time);
    }

    private static void showNaturalOrderExample() {
        List list = new ArrayList();
        list.add("arbeit");
        list.add("nderung");
        list.add("zug");
        list.add("andenken");
        list.add("Arbeit");
        list.add("nderung");
        list.add("Zug");
        list.add("Andenken");
        Collections.sort(list);
        System.out.println("Lexical Ordering:");
        for (Iterator i = list.iterator(); i.hasNext();) {
            System.out.println(i.next());
        }
        System.out.println();
        System.out.println("With Collator:");
        Collections.sort(list, collator);
        for (Iterator i = list.iterator(); i.hasNext();) {
            System.out.println(i.next());
        }
    }

    private static void sort(Object[] freshList) {
        Arrays.sort(freshList);
    }

    private static void sortCaseInsensitive(Object[] freshList) {
        Arrays.sort(freshList, String.CASE_INSENSITIVE_ORDER);
    }

    private static void sortCollator(Object[] freshList) {
        Arrays.sort(freshList, collator);
    }

    private static void sortCombinedCaseSensitive(Object[] freshList) {
        Arrays.sort(freshList);
        Arrays.sort(freshList, collator);
    }

    private static void sortCombinedCaseInsensitive(Object[] freshList) {
        Arrays.sort(freshList, String.CASE_INSENSITIVE_ORDER);
        Arrays.sort(freshList, collator);
    }

    private static void fillWordList(String fileName) throws IOException {
        BufferedReader in = null;
        try {
            in = new BufferedReader(new FileReader(fileName));
            String word;
            while ((word = in.readLine()) != null) {
                words.add(word);
            }
        } finally {
            try {
                in.close();
            } catch (IOException ioe) {
            }
        }
    }

}
