/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.io.*;
import java.util.StringTokenizer;
import java.util.List;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.apache.oro.text.perl.Perl5Util;
import org.apache.regexp.RE;

/**
 * Demonstrates splitting of strings.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class StringSplitDemo {

    private static java.util.regex.Pattern pattern = Pattern.compile(" ");
    private static Perl5Util perl = new Perl5Util();
    private static RE re;

    public static void main(String[] args) throws Exception {
        re = new RE(" ");
        System.out.println("Loading String...");
        String s = loadString(new File(args[0]));
        int iterations = 2000;
        long start;
        long time;

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) stringTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("stringTokenizer first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) stringTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("stringTokenizer second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) singleDelimiterStringTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("singleDelimiterStringTokenizer first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) singleDelimiterStringTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("singleDelimiterStringTokenizer second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) streamTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("streamTokenizer first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) streamTokenizer(s);
        time = System.currentTimeMillis() - start;
        System.out.println("streamTokenizer second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) stringSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("stringSplit first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) stringSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("stringSplit second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) regexSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("regexSplit first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) regexSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("regexSplit second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) oroSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("oroSplit first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) oroSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("oroSplit second run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) reSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("reSplit first run: " + time);

        start = System.currentTimeMillis();
        for (int i=0; i<iterations; i++) reSplit(s);
        time = System.currentTimeMillis() - start;
        System.out.println("reSplit second run: " + time);

    }

    private static String[] stringTokenizer(String s) {
        List list = new ArrayList();
        StringTokenizer st = new StringTokenizer(s, " ");
        while (st.hasMoreTokens()) {
            list.add(st.nextToken());
        }
        return (String[])list.toArray(new String[0]);
    }

    private static String[] singleDelimiterStringTokenizer(String s) {
        List list = new ArrayList();
        SingleDelimiterStringTokenizer st = new SingleDelimiterStringTokenizer(s, ' ');
        while (st.hasMoreTokens()) {
            list.add(st.nextToken());
        }
        return (String[])list.toArray(new String[0]);
    }

    private static String[] streamTokenizer(String s) throws IOException {
        List list = new ArrayList();
        StreamTokenizer st = new StreamTokenizer(new ByteArrayInputStream(s.getBytes()));
        st.eolIsSignificant(false);
        st.slashSlashComments(false);
        st.slashStarComments(false);
        st.whitespaceChars(' ', ' ');
        while (st.nextToken() != StreamTokenizer.TT_EOF) {
            list.add(st.sval);
        }
        return (String[])list.toArray(new String[0]);
    }

    private static String[] stringSplit(String s) throws IOException {
        return s.split(" ");
    }

    private static String[] regexSplit(String s) throws IOException {
        return pattern.split(s);
    }

    private static String[] oroSplit(String s) throws Exception {
        List list = new ArrayList();
        perl.split(list, "/ /", s);
        return (String[])list.toArray(new String[0]);
    }

    private static String[] reSplit(String s) throws Exception {
        return re.split(s);
    }

    private static String loadString(File file) throws IOException {
        BufferedReader in = new BufferedReader(new FileReader(file));
        String line;
        StringBuffer sb = new StringBuffer((int)file.length());
        while ((line = in.readLine()) != null) {
            sb.append(line);
            sb.append(' ');
        }
        in.close();
        return sb.toString();
    }
}
