/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings.helpers;

import java.io.*;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Extracts words from a text.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class WordExtractor {

    private Set set = new HashSet();
    private File inputFile;
    private File outputFile;
    private File upperCaseOutputFile;

    public WordExtractor(String inName) throws IOException {
        inputFile = new File(inName);
        outputFile = new File(inName + ".list");
        upperCaseOutputFile = new File(inName + ".uc_list");
    }

    public void extract() throws IOException {
        readWords();
        writeWords();
        System.out.println("Extracted " + set.size() + " words.");
    }

    private void writeWords() throws IOException {
        PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(outputFile)));
        try {
            for (Iterator i = set.iterator(); i.hasNext();) {
                out.println(i.next());
            }
        } finally {
            out.close();
        }
    }

    private void writeUppercaseWords() throws IOException {
        PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(upperCaseOutputFile)));
        try {
            for (Iterator i = set.iterator(); i.hasNext();) {
                out.println(((String) i.next()).toUpperCase());
            }
        } finally {
            out.close();
        }
    }

    private void readWords() throws IOException {
        BufferedReader in = null;
        try {
            in = new BufferedReader(new FileReader(inputFile));
            char[] cbuf = new char[1024 * 8];
            int length;
            StringBuffer sb = new StringBuffer();
            while ((length = in.read(cbuf)) != -1) {
                for (int i = 0; i < length; i++) {
                    char c = cbuf[i];
                    if (Character.isLetter(c)) {
                        sb.append(c);
                    } else {
                        if (sb.length() > 1) set.add(sb.toString());
                        sb.setLength(0);
                    }
                }
            }
            if (sb.length() > 1) set.add(sb.toString());
        } finally {
            try {
                in.close();
            } catch (IOException ioe) {
            }
        }
    }

    public static void main(String[] args) throws IOException {
        WordExtractor we = new WordExtractor(args[0]);
        we.extract();
    }
}
