/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.threads;

import com.tagtraum.perf.datastructures.MatrixOperations;

import java.util.Random;

/**
 * Uses {@link MatrixOperations}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class MatrixMultiplyDemo {

   private static Random rnd = new Random();

   public static void main(String[] args) throws Exception{
      for (int threads = 1; threads < 65; threads*=2) {
         multiplyWithMultipleDimensions(threads);
      }
   }

   private static void multiplyWithMultipleDimensions(int threads) throws InterruptedException {
      for (int i = 32; i < 513; i += 32) {
         multiply(threads, i);
      }
   }

   private static void multiply(int threads, int i) throws InterruptedException{
      int[][] a = makeRandomMatrix(i);
      int[][] b = makeRandomMatrix(i);
      System.out.print(threads + "," + i + ",");
      int iterations = 10;
      int sum = 0;
      for (int r=0; r<iterations; r++) {
         long start = System.currentTimeMillis();
         MatrixOperations.multithreadedMultiply(a, b, threads);
         sum += (int)(System.currentTimeMillis() - start);
         System.gc();
         Thread.sleep(100);
      }
      a = null;
      b = null;
      System.gc();
      Thread.sleep(100);
      System.out.println((sum/iterations));
   }

   private static int[][] makeRandomMatrix(int dim) {
      int[][] a = new int[dim][dim];
      for (int i = 0; i < a.length; i++) {
         for (int j = 0; j < a.length; j++) {
            a[i][j] = rnd.nextInt(10) - 5;
         }
      }
      return a;
   }
}
