/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.threads;

/**
 * Demonstrates the costs of notify/notifyAll.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class NotifyDemo {
   public static void main(String[] args) throws InterruptedException {
      long start;

      notifyAllPingPong();

      NotifyPingPong ping;
      NotifyPingPong pong;

      ping = new NotifyPingPong(true);
      pong = new NotifyPingPong(false);
      ping.setPingPong(pong);
      pong.setPingPong(ping);

      start = System.currentTimeMillis();
      pong.start();
      ping.start();
      pong.join();
      ping.join();
      System.out.println("notify: " + (System.currentTimeMillis() - start));

      ping = new NotifyPingPong(true);
      pong = new NotifyPingPong(false);
      ping.setPingPong(pong);
      pong.setPingPong(ping);

      start = System.currentTimeMillis();
      pong.start();
      ping.start();
      pong.join();
      ping.join();
      System.out.println("notify: " + (System.currentTimeMillis() - start));
   }

   private static void notifyAllPingPong() throws InterruptedException {
      long start;
      NotifyAllPingPong ping = new NotifyAllPingPong(true);
      NotifyAllPingPong pong = new NotifyAllPingPong(false);
      ping.setPingPong(pong);
      pong.setPingPong(ping);

      start = System.currentTimeMillis();
      pong.start();
      ping.start();
      pong.join();
      ping.join();
      System.out.println("notifyAll: " + (System.currentTimeMillis() - start));

      ping = new NotifyAllPingPong(true);
      pong = new NotifyAllPingPong(false);
      ping.setPingPong(pong);
      pong.setPingPong(ping);

      start = System.currentTimeMillis();
      pong.start();
      ping.start();
      pong.join();
      ping.join();
      System.out.println("notifyAll: " + (System.currentTimeMillis() - start));
   }

   private static class NotifyAllPingPong extends Thread {
      Object lock = new Object();
      boolean condition;
      int number;
      NotifyAllPingPong other;

      public NotifyAllPingPong(boolean condition) {
         setCondition(condition);
      }

      public void setPingPong(NotifyAllPingPong other) {
         this.other = other;
      }

      public void waitForCondition()
            throws InterruptedException {
         synchronized (lock) {
            while (condition) {
               lock.wait();
            }
            condition = true;
         }
      }

      public void setCondition(boolean condition) {
         synchronized (lock) {
            if (this.condition != condition) {
            this.condition = condition;
            number++;
            // Alle wartenden Threads verstndigen.
            lock.notifyAll();
         }
         }
      }

      public void run() {
         try {
         for (int i=0; i<100000; i++) {
            waitForCondition();
            other.setCondition(false);
         }
         }
         catch (InterruptedException ie) {
            ie.printStackTrace();
         }
      }
   }
   private static class NotifyPingPong extends Thread {
      Object lock = new Object();
      boolean condition;
      int number;
      NotifyPingPong other;

      public NotifyPingPong(boolean condition) {
         setCondition(condition);
      }

      public void setPingPong(NotifyPingPong other) {
         this.other = other;
      }

      public void waitForCondition()
            throws InterruptedException {
         synchronized (lock) {
            while (condition) {
               lock.wait();
            }
            condition = true;
         }
      }

      public void setCondition(boolean condition) {
         synchronized (lock) {
            if (this.condition != condition) {
            this.condition = condition;
            number++;
            // Alle wartenden Threads verstndigen.
            lock.notify();
         }
         }
      }

      public void run() {
         try {
         for (int i=0; i<100000; i++) {
            waitForCondition();
            other.setCondition(false);
         }
         }
         catch (InterruptedException ie) {
            ie.printStackTrace();
         }
      }
   }
}
