/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.threads;

/**
 * Demonstrates the effects of sychronized.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class SynchronizedDemo {
   private static UnsynchronizedTime unsynchronizedTime = new UnsynchronizedTime();
   private static SynchronizedTime synchronizedTime = new SynchronizedTime();


   public static void main(String[] args) {
      long start = 0;

      start = System.currentTimeMillis();
      setUnsynchronizedTime();
      System.out.println("setUnsynchronizedTime(): " + (System.currentTimeMillis() - start));

      start = System.currentTimeMillis();
      setUnsynchronizedTime();
      System.out.println("setUnsynchronizedTime(): " + (System.currentTimeMillis() - start));

      start = System.currentTimeMillis();
      synchronized (unsynchronizedTime) {
         setUnsynchronizedTime();
      }
      System.out.println("synchronized () {setUnsynchronizedTime()}: " + (System.currentTimeMillis() - start));

      start = System.currentTimeMillis();
      synchronized (unsynchronizedTime) {
         setUnsynchronizedTime();
      }
      System.out.println("synchronized () {setUnsynchronizedTime()}: " + (System.currentTimeMillis() - start));

      start = System.currentTimeMillis();
      setSynchronizedTime();
      System.out.println("setSynchronizedTime(): " + (System.currentTimeMillis() - start));

      start = System.currentTimeMillis();
      setSynchronizedTime();
      System.out.println("setSynchronizedTime(): " + (System.currentTimeMillis() - start));
   }

   private static void setUnsynchronizedTime() {
      for (int i = 0; i < 500000000; i++) unsynchronizedTime.setTime(i);
   }

   private static void setSynchronizedTime() {
      for (int i = 0; i < 500000000; i++) synchronizedTime.setTime(i);
   }

   private static class UnsynchronizedTime {
      private long time;

      public void setTime(long time) {
         this.time = time;
      }
   }

   private static class SynchronizedTime {
      private long time;

      public synchronized void setTime(long time) {
         this.time = time;
      }
   }
}
