/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.threads;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * Loads a URL in an extra thread.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 * @see URLLoaderDemo2
 */
public class URLLoaderDemo2 extends Frame {

   private TextField urlField;
   private TextArea textArea;
   private Label statusBar;

   public static void main(String[] args) {
      new URLLoaderDemo2();
   }

   public URLLoaderDemo2() {
      super("URLLoaderDemo");
      Panel panel = new Panel();
      Button loadButton = new Button("Load");
      urlField = new TextField("Enter URL here", 30);
      textArea = new TextArea(20, 65);
      statusBar = new Label();
      final URLLoader urlLoader = new URLLoader();

      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            dispose();
            urlLoader.cancel();
         }
      });
      loadButton.addActionListener(urlLoader);
      urlField.addActionListener(urlLoader);

      panel.add(urlField, BorderLayout.NORTH);
      panel.add(loadButton, BorderLayout.NORTH);
      add(panel, BorderLayout.NORTH);
      add(textArea, BorderLayout.CENTER);
      add(statusBar, BorderLayout.SOUTH);

      setSize(300, 300);
      pack();
      setVisible(true);
   }

   private class URLLoader implements ActionListener, Runnable {
      private Thread t;
      private URL url;

      public void actionPerformed(ActionEvent e) {
         try {
            cancel();
            url = new URL(urlField.getText());
            t = new Thread(this);
            t.start();
         } catch (MalformedURLException mfue) {
            statusBar.setText(mfue.toString());
         }
      }

      public void cancel() {
         if (t != null && t.isAlive()) {
            try {
               t.interrupt();
               t.join();
               t = null;
            } catch (InterruptedException ie) {
               statusBar.setText(ie.toString());
            }
         }
      }

      public void run() {
         textArea.setText("");
         repaint();
         Reader in = null;
         statusBar.setText("Loading " + url);
         char[] cbuf = new char[1024];
         try {
            URLConnection connection = url.openConnection();
            in = new InputStreamReader(connection.getInputStream());
            int contentLength = connection.getContentLength();
            long lastRepaint = 0;
            for (int count = 0, sum = 0; (count = in.read(cbuf)) != -1 && !t.isInterrupted();) {
               String s = new String(cbuf, 0, count);
               textArea.append(s);
               sum += count;
               showProgress(contentLength, sum);
               if (System.currentTimeMillis() - lastRepaint > 500) {
                  repaint();
                  lastRepaint = System.currentTimeMillis();
               }
            }
            if (t.isInterrupted())
               statusBar.setText("Interrupted.");
            else
               statusBar.setText("Done.");
         } catch (IOException ioe) {
            statusBar.setText(ioe.toString());
         } finally {
            if (in != null)
               try {
                  in.close();
               } catch (IOException ioe) {
               }
         }
         repaint();
      }

      private void showProgress(int contentLength, int sum) {
         if (contentLength != -1) {
            statusBar.setText((sum / 1024) + "K/" + (contentLength / 1024) + "K - " + (sum * 100 / (contentLength)) + "%");
         } else {
            statusBar.setText((sum / 1024) + "K");
         }
      }
   }
}
