/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.xml;

import com.tagtraum.perf.io.HuffmanOutputStream;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import java.io.*;
import java.util.zip.ZipOutputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.xml.sax.helpers.DefaultHandler;

/**
 * Demonstrates the compression of XML in different formats.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class BSAXCompressionDemo {
   public static void main(String[] args) throws Exception {
      File file = new File(args[0]);
      bsaxOnly(file);
      bsaxOnly(file);
      bsaxZip(file);
      bsaxZip(file);
      bsaxGZip(file);
      bsaxGZip(file);
      bsaxHuffman(file);
      bsaxHuffman(file);
      zip(file);
      zip(file);
      gzip(file);
      gzip(file);
      huffman(file);
      huffman(file);
   }

   public static void bsaxOnly(File file) throws Exception {
      File out = File.createTempFile("test", "bsax");
      OutputStream outStream = new FileOutputStream(out);
      BSAXEncoder encoder = new BSAXEncoder(outStream);
      encoder.setIgnoreIgnorableWhitespace(true);
      SAXParserFactory factory = SAXParserFactory.newInstance();
      SAXParser parser = factory.newSAXParser();
      long start = System.currentTimeMillis();
      parser.parse(file, encoder);
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void bsaxZip(File file) throws Exception {
      File out = File.createTempFile("test", "bsax-zip");
      ZipOutputStream outStream = new ZipOutputStream(new FileOutputStream(out));
      outStream.putNextEntry(new ZipEntry("bsax"));
      BSAXEncoder encoder = new BSAXEncoder(outStream);
      SAXParserFactory factory = SAXParserFactory.newInstance();
      SAXParser parser = factory.newSAXParser();
      long start = System.currentTimeMillis();
      parser.parse(file, encoder);
      outStream.closeEntry();
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void bsaxGZip(File file) throws Exception {
      File out = File.createTempFile("test", "bsax-gz");
      GZIPOutputStream outStream = new GZIPOutputStream(new FileOutputStream(out));
      BSAXEncoder encoder = new BSAXEncoder(outStream);
      encoder.setIgnoreIgnorableWhitespace(true);
      SAXParserFactory factory = SAXParserFactory.newInstance();
      SAXParser parser = factory.newSAXParser();
      long start = System.currentTimeMillis();
      parser.parse(file, encoder);
      outStream.finish();
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void bsaxHuffman(File file) throws Exception {
      File out = File.createTempFile("test", "bsax-huff");
      OutputStream outStream = new HuffmanOutputStream(new FileOutputStream(out));
      BSAXEncoder encoder = new BSAXEncoder(outStream);
      SAXParserFactory factory = SAXParserFactory.newInstance();
      SAXParser parser = factory.newSAXParser();
      long start = System.currentTimeMillis();
      parser.parse(file, encoder);
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void zip(File file) throws Exception {
      File out = File.createTempFile("test", "zip");
      long start = System.currentTimeMillis();
      byte[] buf = new byte[8*1024];
      ZipOutputStream outStream = new ZipOutputStream(new FileOutputStream(out));
      outStream.putNextEntry(new ZipEntry("bsax"));
      InputStream in = new FileInputStream(file);
      int len = 0;
      while ((len = in.read(buf)) != -1) {
         outStream.write(buf, 0, len);
      }
      in.close();
      outStream.closeEntry();
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void gzip(File file) throws Exception {
      File out = File.createTempFile("test", "gz");
      long start = System.currentTimeMillis();
      byte[] buf = new byte[8*1024];
      GZIPOutputStream outStream = new GZIPOutputStream(new FileOutputStream(out));
      InputStream in = new FileInputStream(file);
      int len = 0;
      while ((len = in.read(buf)) != -1) {
         outStream.write(buf, 0, len);
      }
      in.close();
      outStream.finish();
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

   public static void huffman(File file) throws Exception {
      File out = File.createTempFile("test", "huff");
      long start = System.currentTimeMillis();
      byte[] buf = new byte[8*1024];
      OutputStream outStream = new HuffmanOutputStream(new FileOutputStream(out));
      InputStream in = new FileInputStream(file);
      int len = 0;
      while ((len = in.read(buf)) != -1) {
         outStream.write(buf, 0, len);
      }
      in.close();
      outStream.close();
      System.out.println(System.currentTimeMillis()-start);
      System.out.println(out);
      System.out.println(file.length() + " -> " + out.length() + " " + (out.length()*100/file.length()));
   }

}

