/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.xml;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;

/**
 * Demonstrates parsing a document using a DOM parser and a recursive traversal method.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class DOMBenchDemo2 {
   public static void main(String[] args) throws Exception {
      DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
      // adjust to your needs:
      //builderFactory.setAttribute("http://apache.org/xml/features/dom/defer-node-expansion", Boolean.FALSE);
      //builderFactory.setValidating(false);
      //builderFactory.setNamespaceAware(false);
      DocumentBuilder documentBuilder = builderFactory.newDocumentBuilder();
      File file = new File(args[0]);
      parse(documentBuilder, file);
      parse(documentBuilder, file);
   }

   private static void parse(DocumentBuilder documentBuilder, File file) throws Exception {
      //long start = System.currentTimeMillis();
      Document document = documentBuilder.parse(file);
      long start = System.currentTimeMillis();
      traverse(document.getDocumentElement());
      System.out.println(System.currentTimeMillis() - start);
   }

   private static void traverse(Element element) throws Exception {
      element.getTagName();
      NodeList list = element.getChildNodes();
      for (int i = 0, length = list.getLength(); i < length; i++) {
         if (list.item(i).getNodeType() == Node.ELEMENT_NODE) traverse((Element) list.item(i));
      }
   }

}
