/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * ArrayList based matrix. Suitable for small matrices.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ArrayMatrix extends AbstractMatrix implements Serializable {

   private List listOfRowLists = new ArrayList();

   public ArrayMatrix(int cols, int rows) {
      super(cols, rows);
   }

   public Object set(int col, int row, Object object) {
      if (object == null) return remove(col, row);
      rangeCheck(col, row);
      List rowList = (List) listGet(listOfRowLists, row);
      if (rowList == null) {
         rowList = new ArrayList();
         listSet(listOfRowLists, row, rowList);
      }
      return listSet(rowList, col, object);
   }

   public Object get(int col, int row) {
      rangeCheck(col, row);
      List rowList = (List) listGet(listOfRowLists, row);
      if (rowList != null) {
         return listGet(rowList, col);
      }
      return null;
   }

   public Object remove(int col, int row) {
      rangeCheck(col, row);
      ArrayList rowList = (ArrayList) listGet(listOfRowLists, row);
      Object oldValue = null;
      if (rowList != null) {
         oldValue = listRemove(rowList, col);
         trimListToSize(rowList, row);
      }
      return oldValue;
   }

   private void trimListToSize(ArrayList rowList, int row) {
      while (!rowList.isEmpty() && rowList.get(rowList.size() - 1) == null) {
         rowList.remove(rowList.size() - 1);
      }
      if (rowList.isEmpty())
         listOfRowLists.set(row, null);
      else
         rowList.trimToSize();
   }

   private Object listSet(List list, int index, Object object) {
      while (list.size() - 1 < index) list.add(null);
      return list.set(index, object);
   }

   private Object listGet(List list, int index) {
      if (list.size() <= index) return null;
      return list.get(index);
   }

   private Object listRemove(List list, int index) {
      if (list.size() <= index) return null;
      return list.set(index, null);
   }

}
