/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;


/**
 * Interface for caches.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public interface Cache {

   /**
    * Returns the capacity of this cache. The exact meaning of this value is
    * implementation dependent.
    */
   public int getCapacity();

   /**
    * Retrieves an object from this cache using the given key.
    *
    * @return null if the object is not in this cache (anymore)
    */
   public Object get(Object key);

   /**
    * Registers an object under a key in this cache.
    */
   public Object put(Object key, Object value);

   /**
    * Returns a number between 0.0 and 1.0. 1.0 is a 100% hit rate.
    */
   public float getHitRatio();
}
