/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * HashMap based matrix. Suitable for sparse matrices. Unsynchronized.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class HashMatrix extends AbstractMatrix implements Serializable {

   private Map map;
   private boolean growable;
   private Key searchKey;

   public HashMatrix(int cols, int rows) {
      this(cols, rows, false);
   }

   public HashMatrix(int cols, int rows, boolean growable) {
      super(cols, rows);
      map = new HashMap();
      setGrowable(growable);
      searchKey = new Key(0, 0);
   }

   public Object set(int col, int row, Object object) {
      rangeCheck(col, row);
      return map.put(new Key(col, row), object);
   }

   public Object get(int col, int row) {
      rangeCheck(col, row);
      return map.get(searchKey.set(col, row));
   }

   public Object remove(int col, int row) {
      rangeCheck(col, row);
      return map.remove(searchKey.set(col, row));
   }

   private static class Key {
      int col;
      int row;
      int hashCode;

      public Key(int col, int row) {
         set(col, row);
      }

      private Key set(int col, int row) {
         this.row = row;
         this.col = col;
         this.hashCode = col ^ -row;
         return this;
      }

      public int hashCode() {
         return hashCode;
      }

      public boolean equals(Object obj) {
         Key other = (Key) obj;
         return other.col == col && other.row == row;
      }

      public String toString() {
         return col + ":" + row;
      }
   }
}
