/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

/**
 * Matrix that can hold objects, not just primitive values.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public interface Matrix {

   /**
    * Sets an Object at a given position.
    *
    * @param col column
    * @param row row
    * @param object Object to set
    * @return the object that was replaced or <code>null</code> if there was none
    * @exception IndexOutOfBoundsException if col or row are less than zero or greater
    *    than the size of this matrix
    */
   public Object set(int col, int row, Object object);

   /**
    * Returns an Object from a given position.
    *
    * @param col column
    * @param row row
    * @return the object that is at the given position or <code>null</code> if there is none
    * @exception IndexOutOfBoundsException if col or row are less than zero or greater
    *    than the size of this matrix
    */
   public Object get(int col, int row);

   /**
    * Removes an Object from a given position.
    *
    * @param col column
    * @param row row
    * @return the object that was at the given position or <code>null</code> if there was none
    * @exception IndexOutOfBoundsException if col or row are less than zero or greater
    *    than the size of this matrix
    */
   public Object remove(int col, int row);

   /**
    * The number of columns this matrix has.
    *
    * @return the number of columns
    */
   public int columns();

   /**
    * The number of rows this matrix has.
    *
    * @return the number of rows
    */
   public int rows();

   /**
    * Indicates whether this matrix can grow or has a fixed size.
    */
   public boolean isGrowable();

}
