/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.io.IOException;
import java.io.Serializable;
import java.text.Collator;
import java.util.*;

/**
 * Map that wraps keys in CollationsKeys.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class CollationKeyMap implements Map, Serializable {
    private Map map;
    private Locale locale;
    private int collatorStrength;
    private transient Collator collator;

    public CollationKeyMap(Locale locale, int collatorStrength) {
        this.map = new HashMap();
        this.locale = locale == null ? Locale.getDefault() : locale;
        this.collatorStrength = collatorStrength;
        this.collator = Collator.getInstance(this.locale);
        this.collator.setStrength(this.collatorStrength);
    }

    // string specific methods

    public Object get(String key) {
        if (key == null) return map.get(null);
        return map.get(collator.getCollationKey(key));
    }

    public Object remove(String key) {
        if (key == null) return map.remove(null);
        return map.get(collator.getCollationKey(key));
    }

    public Object put(String key, Object value) {
        if (key == null) return map.put(null, value);
        return map.put(collator.getCollationKey(key), value);
    }

    public boolean containsKey(String key) {
        if (key == null) return map.containsKey(null);
        return map.containsKey(collator.getCollationKey(key));
    }

    // other methods

    public int size() {
        return map.size();
    }

    public boolean isEmpty() {
        return map.isEmpty();
    }

    public boolean containsKey(Object key) {
        return containsKey((String) key);
    }

    public boolean containsValue(Object value) {
        return map.containsValue(value);
    }

    public Object get(Object key) {
        return get((String) key);
    }

    public Object put(Object key, Object value) {
        return put((String) key, value);
    }

    public Object remove(Object key) {
        return remove((String) key);
    }

    public void putAll(Map t) {
        Iterator i = t.entrySet().iterator();
        while (i.hasNext()) {
            Map.Entry e = (Map.Entry) i.next();
            put(e.getKey(), e.getValue());
        }
    }

    public void clear() {
        map.clear();
    }

    public Set keySet() {
        return map.keySet();
    }

    public Collection values() {
        return map.values();
    }

    public Set entrySet() {
        return map.entrySet();
    }

    public int hashCode() {
        return map.hashCode();
    }

    public boolean equals(Object obj) {
        return map.equals(obj);
    }

    public String toString() {
        return map.toString();
    }

    private void readObject(java.io.ObjectInputStream in)
            throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        collator = Collator.getInstance(locale);
        collator.setStrength(collatorStrength);
    }

}
