/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.swing;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import java.util.List;

/**
 * ArrayList based TableModel.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ArrayListTableModel extends AbstractTableModel {

   private int cols;
   private int rows;
   private ArrayList listOfRowLists;

   public ArrayListTableModel(int rows, int cols) {
      listOfRowLists = new ArrayList();
      this.rows = rows;
      this.cols = cols;
   }

   public int getRowCount() {
      return rows;
   }

   public int getColumnCount() {
      return cols;
   }

   public Object getValueAt(int row, int col) {
      List rowList = (List) get(listOfRowLists, row);
      if (rowList != null) {
         return get(rowList, col);
      }
      return null;
   }

   public void setValueAt(Object object, int row, int col) {
      if (object == null) {
         removeValueAt(col, row);
      } else {
         ArrayList rowList = (ArrayList) get(listOfRowLists, row);
         if (rowList == null) {
            rowList = new ArrayList();
            set(listOfRowLists, row, rowList);
         }
         set(rowList, col, object);
      }
      fireTableCellUpdated(row, col);
   }

   private void removeValueAt(int row, int col) {
      ArrayList rowList = (ArrayList) get(listOfRowLists, row);
      if (rowList != null) {
         remove(rowList, col);
      }
   }

   private void set(ArrayList list, int index, Object object) {
      while (list.size() - 1 < index) list.add(null);
      list.set(index, object);
      list.trimToSize();
   }

   private Object get(List list, int index) {
      if (list.size() <= index) return null;
      return list.get(index);
   }

   private void remove(ArrayList list, int index) {
      if (list.size() > index) list.set(index, null);
      while (!list.isEmpty() && list.get(list.size() - 1) == null) {
         list.remove(list.size() - 1);
      }
      if (list.isEmpty())
         listOfRowLists.set(index, null);
      else
         list.trimToSize();
   }

}
