/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.swing;

import javax.swing.table.AbstractTableModel;
import java.util.HashMap;
import java.util.Map;

/**
 * Hashbased TableModel suitable for sparse TableModels.
 * Note that this implementation is not synchronized.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class HashTableModel extends AbstractTableModel {

   private int cols;
   private int rows;
   private Map map;
   private Key searchKey;

   public HashTableModel(int rows, int cols) {
      map = new HashMap();
      this.rows = rows;
      this.cols = cols;
      searchKey = new Key(0, 0);
   }

   public int getRowCount() {
      return rows;
   }

   public int getColumnCount() {
      return cols;
   }

   public Object getValueAt(int row, int col) {
      return map.get(searchKey.set(row, col));
   }

   public void setValueAt(Object object, int row, int col) {
      if (object == null) {
         if (map.remove(searchKey.set(row, col)) != null) {
            fireTableCellUpdated(row, col);
         }
      }
      else {
         if (map.put(new Key(row, col), object) != object) {
            if (row >= rows && col < cols) {
               rows = row;
               if (col < cols) {
                  fireTableDataChanged();
               }
               else {
                  cols = col;
                  fireTableStructureChanged();
               }
            }
            else fireTableCellUpdated(row, col);
         }
      }
   }

   public boolean isCellEditable(int row, int col) {
      return true;
   }

   private static class Key {
      int col;
      int row;
      int hashCode;

      public Key(int row, int col) {
         set(row, col);
      }

      public Key set(int row, int col) {
         this.row = row;
         this.col = col;
         this.hashCode = col ^ -row;
         return this;
      }

      public int hashCode() {
         return hashCode;
      }

      public boolean equals(Object obj) {
         Key other = (Key) obj;
         return other.col == col && other.row == row;
      }

      public String toString() {
         return col + ":" + row;
      }
   }
}
