/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.swing;

import com.tagtraum.perf.datastructures.Matrix;
import com.tagtraum.perf.datastructures.HashMatrix;

import javax.swing.table.AbstractTableModel;

/**
 * TableModel backed by a {@link Matrix}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class MatrixTableModel extends AbstractTableModel {

   private Matrix m;

   public MatrixTableModel(int cols, int rows) {
      m = new HashMatrix(cols, rows);
   }

   public MatrixTableModel(Matrix m) {
      if (m == null) throw new NullPointerException();
      this.m = m;
   }

   public int getRowCount() {
      return m.rows();
   }

   public int getColumnCount() {
      return m.columns();
   }

   public Object getValueAt(int rowIndex, int columnIndex) {
      return m.get(columnIndex, rowIndex);
   }

   public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
      Object oldValue = m.set(columnIndex, rowIndex, aValue);
      if (oldValue != aValue) fireTableCellUpdated(rowIndex, columnIndex);
   }

}
