/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * Tests {@link CRSMatrix}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestCRSMatrix extends TestCase {

   public TestCRSMatrix(String name) {
      super(name);
   }

   public void testSetGet() {
      Matrix m = new CRSMatrix(10, 10);
      for (int col=0; col<m.columns(); col++) {
         for (int row=0; row<m.rows(); row++) {
            m.set(col, row, col + ":" + row);
         }
      }
      for (int col=0; col<m.columns(); col++) {
         for (int row=0; row<m.rows(); row++) {
            assertEquals(col + ":" + row, m.get(col, row));
         }
      }
   }

   public void testSetRemoveGet() {
      Matrix m = new CRSMatrix(10, 10);
      for (int col=0; col<m.columns(); col++) {
         for (int row=0; row<m.rows(); row++) {
            assertNull(m.set(col, row, col + ":" + row));
         }
      }
      for (int col=0; col<m.columns(); col++) {
         for (int row=0; row<m.rows(); row++) {
            assertEquals(col + ":" + row, m.remove(col, row));
         }
      }
      for (int col=0; col<m.columns(); col++) {
         for (int row=0; row<m.rows(); row++) {
            assertNull(m.get(col, row));
         }
      }
   }

   public static TestSuite suite() {
      return new TestSuite(TestCRSMatrix.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
