/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestIntegerSortPipe1 extends TestCase {

   public TestIntegerSortPipe1(String name) {
      super(name);
   }

   public void test() throws IOException {
      int[] orderedArray = getIntArray(5);
      int[] shuffledArray = (int[]) orderedArray.clone();
      Arrays.shuffle(shuffledArray);

      IntegerSortPipe1 bsb = new IntegerSortPipe1(orderedArray.length, 0);

      ByteBuffer in = ByteBuffer.wrap(new byte[orderedArray.length * 4]);
      for (int i = 0; in.hasRemaining(); i++) {
         in.putInt(shuffledArray[i]);
      }
      in.rewind();
      bsb.write(in);

      ByteBuffer out = ByteBuffer.wrap(new byte[orderedArray.length * 4]);
      bsb.read(out);
      out.rewind();
      for (int i = 0; out.hasRemaining(); i++) {
         assertEquals(orderedArray[i], out.getInt());
      }
   }

   private int[] getIntArray(int limit) {
      int[] arr = new int[limit];
      for (int i = 0; i < limit; i++) {
         arr[i] = i;
      }
      return arr;
   }


   public static TestSuite suite() {
      return new TestSuite(TestIntegerSortPipe1.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
