/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.Random;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestHuffmanInputStream extends TestCase {

   public TestHuffmanInputStream(String name) {
      super(name);
   }

   // random numbers
   public void testReadArrayWithHuffmanOutputStream() throws Exception {
      ByteArrayOutputStream bout = new ByteArrayOutputStream();
      HuffmanOutputStream out = new HuffmanOutputStream(bout);
      byte[] buf = new byte[1024 * 100];
      Random rnd = new Random();
      rnd.nextBytes(buf);
      out.write(buf);
      out.close();

      byte[] rbuf = new byte[buf.length];
      HuffmanInputStream in = new HuffmanInputStream(new ByteArrayInputStream(bout.toByteArray()));
      in.read(rbuf);
      for (int i = 0; i < buf.length; i++) {
         assertEquals(buf[i], rbuf[i]);
      }
   }

   // random numbers
   public void testReadWithHuffmanOutputStream() throws Exception {
      ByteArrayOutputStream bout = new ByteArrayOutputStream();
      HuffmanOutputStream out = new HuffmanOutputStream(bout);
      byte[] buf = new byte[1024 * 100];
      Random rnd = new Random();
      rnd.nextBytes(buf);
      out.write(buf);
      out.close();

      HuffmanInputStream in = new HuffmanInputStream(new ByteArrayInputStream(bout.toByteArray()));
      for (int i = 0; i < buf.length; i++) {
         assertEquals(buf[i], (byte) in.read());
      }
   }

   // non-random numbers
   public void testReadArrayWithHuffmanOutputStream2() throws Exception {
      ByteArrayOutputStream bout = new ByteArrayOutputStream();
      HuffmanOutputStream out = new HuffmanOutputStream(bout);
      byte[] buf = new byte[1024 * 100];
      for (int i = 0; i < buf.length; i++) {
         buf[i] = (byte) i;
      }
      out.write(buf);
      out.close();

      byte[] rbuf = new byte[buf.length];
      HuffmanInputStream in = new HuffmanInputStream(new ByteArrayInputStream(bout.toByteArray()));
      in.read(rbuf);
      for (int i = 0; i < buf.length; i++) {
         assertEquals(buf[i], rbuf[i]);
      }
   }


   public static TestSuite suite() {
      return new TestSuite(TestHuffmanInputStream.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
