/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.util;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.util.Random;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestHuffmanTree extends TestCase {

   public TestHuffmanTree(String name) {
      super(name);
   }

   public void testDummyEncode() throws Exception {
      HuffmanTree huffman = new HuffmanTree();
      byte[] plaintext = new byte[] {'T', 'E', 'S', 'T'};
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      huffman.encode(plaintext, 0, plaintext.length, out);
   }

   public void testSimpleEncodeDecode() throws Exception {
      HuffmanTree huffman = new HuffmanTree();
      byte[] plaintext = new byte[] {'T', 'E', 'S', 'T'};
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      huffman.encode(plaintext, 0, plaintext.length, out);
      ByteArrayInputStream in = new ByteArrayInputStream(out.toByteArray());
      byte[] decodedPlaintext = huffman.decode(in);
      for (int i=0; i<plaintext.length; i++) {
         assertEquals(plaintext[i], decodedPlaintext[i]);
      }
   }

   public void testLargeEncodeDecode() throws Exception {
      HuffmanTree huffman = new HuffmanTree();
      Random rnd = new Random();
      byte[] plaintext = new byte[1024*100];
      rnd.nextBytes(plaintext);
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      huffman.encode(plaintext, 0, plaintext.length, out);
      ByteArrayInputStream in = new ByteArrayInputStream(out.toByteArray());
      byte[] decodedPlaintext = huffman.decode(in);
      for (int i=0; i<plaintext.length; i++) {
         assertEquals(plaintext[i], decodedPlaintext[i]);
      }
   }

   public static TestSuite
         suite() {
      return new TestSuite(TestHuffmanTree.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }

}
