/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.xml;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import com.tagtraum.perf.threads.Runner;
import com.tagtraum.perf.io.NirvanaOutputStream;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;

import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestBSAXEncoder extends TestCase {

   public TestBSAXEncoder(String name) {
      super(name);
   }

   /**
    * Just write some stuff without checking.
    */
   public void testDummyWrite() throws Exception{
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      BSAXEncoder encoder = new BSAXEncoder(out, 8 * 1024); // 8k blocks
      char[] characters = new char[] {'T', 'E', 'S', 'T'};
      char[] whitespace = new char[] {' ', '\t', '\r', '\n'};
      AttributesImpl attr = new AttributesImpl();
      attr.addAttribute("uri", "localName", "qName", "type", "value");
      encoder.startDocument();
      for (int i=0; i<1000; i++) {
         encoder.ignorableWhitespace(whitespace, 0, whitespace.length);
         encoder.startElement("uri", "localName", "qName", attr);
         encoder.characters(characters, 0, characters.length);
         encoder.skippedEntity("skippedEntity");
         encoder.unparsedEntityDecl("unparsedEntityDecl", "publicId", "systemId", "notationName");
         encoder.startPrefixMapping("prefix", "uri");
         encoder.endPrefixMapping("prefix");
         encoder.notationDecl("name", "publicId", "systemId");
         encoder.endElement("uri", "localName", "qName");
         encoder.startElement("uri", "localName", "qName", null);
         encoder.endElement("uri", "localName", "qName");
      }
      encoder.endDocument();
   }

   /**
    * Just write some stuff with checking.
    */
   public void testWithBSAXDecoder() throws Exception{
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      BSAXEncoder encoder = new BSAXEncoder(out, 8 * 1024); // 8k blocks
      char[] characters = new char[] {'T', 'E', 'S', 'T'};
      char[] whitespace = new char[] {' ', '\t', '\r', '\n'};
      AttributesImpl attr = new AttributesImpl();
      attr.addAttribute("uri", "localName", "qName", "type", "value");
      encoder.startDocument();
      for (int i=0; i<1; i++) {
         encoder.ignorableWhitespace(whitespace, 0, whitespace.length);
         encoder.startElement("uri", "localName", "qName", attr);
         encoder.characters(characters, 0, characters.length);
         encoder.skippedEntity("skippedEntity");
         encoder.unparsedEntityDecl("unparsedEntityDecl", "publicId", "systemId", "notationName");
         encoder.startPrefixMapping("prefix", "uri");
         encoder.endPrefixMapping("prefix");
         encoder.notationDecl("name", "publicId", "systemId");
         encoder.endElement("uri", "localName", "qName");
      }
      encoder.endDocument();

      BSAXDecoder decoder = new BSAXDecoder();
      decoder.setContentHandler(new DefaultHandler());
      decoder.parse(new InputSource(new ByteArrayInputStream(out.toByteArray())));
   }

   public static TestSuite
      suite() {
      return new TestSuite(TestBSAXEncoder.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }

}
